import React, { useState, useEffect } from "react";
import { bytesToHex } from "@noble/hashes/utils";
import { MessageId } from "@canvas-js/gossiplog";
export const AppInfo = ({ app, ws, styles = {}, buttonStyles = {}, popupStyles = {} }) => {
    const [, setTick] = useState(0);
    const [isOpen, setIsOpen] = useState(false);
    const [remoteInfo, setRemoteInfo] = useState(null);
    const [isLoadingInfo, setIsLoadingInfo] = useState(false);
    // Add effect to load remote info on mount
    useEffect(() => {
        if (ws === null || ws === undefined)
            return;
        const fetchRemoteInfo = async () => {
            setIsLoadingInfo(true);
            try {
                const response = await fetch(ws.sourceURL.replace(/^ws(s)?:\/\//, "http$1://") + "/api");
                const data = await response.json();
                setRemoteInfo(data);
            }
            catch (error) {
                console.error("Failed to fetch remote info:", error);
                setRemoteInfo({ error: "Failed to fetch remote info" });
            }
            setIsLoadingInfo(false);
        };
        fetchRemoteInfo();
    }, [ws?.sourceURL]);
    useEffect(() => {
        const timer = setInterval(() => {
            setTick((t) => t + 1);
        }, 1000);
        return () => clearInterval(timer);
    }, []);
    // Add escape key handler
    useEffect(() => {
        const handleEscape = (event) => {
            if (event.key === "Escape") {
                setIsOpen(!isOpen);
            }
        };
        document.addEventListener("keydown", handleEscape);
        return () => document.removeEventListener("keydown", handleEscape);
    }, [isOpen]);
    // Get root and heads
    const [root, setRoot] = useState(null);
    const [heads, setHeads] = useState(null);
    useEffect(() => {
        if (app === undefined || app === null) {
            return;
        }
        app.messageLog.tree.read((txn) => txn.getRoot()).then((root) => setRoot(`${root.level}:${bytesToHex(root.hash)}`));
        app.db.query("$heads").then((records) => setHeads(records.map((record) => record.id)));
        const handleCommit = ({ detail: { root, heads } }) => {
            const rootValue = `${root.level}:${bytesToHex(root.hash)}`;
            setRoot(rootValue);
            setHeads(heads);
        };
        app.addEventListener("commit", handleCommit);
        return () => app.removeEventListener("commit", handleCommit);
    }, [app]);
    if (!app)
        return;
    return (React.createElement(React.Fragment, null,
        React.createElement("div", { style: { position: "fixed", top: "0.75rem", right: "1rem", ...styles } },
            React.createElement("div", { style: {
                    border: "1px solid #ddd",
                    borderRadius: "0.25rem",
                    padding: "0.25rem 0.5rem",
                    cursor: "pointer",
                    backgroundColor: "white",
                    color: "#222",
                    ...buttonStyles,
                }, onClick: () => setIsOpen(!isOpen) }, "Info")),
        isOpen && (React.createElement("div", { style: {
                position: "fixed",
                top: "1.25rem",
                right: "1.75rem",
                fontSize: "0.875rem",
                zIndex: 9999,
                ...popupStyles,
            } },
            React.createElement("div", { style: { position: "absolute", top: "0.75rem", right: "0.75rem" } },
                React.createElement("button", { onClick: () => setIsOpen(false), className: "text-gray-500 hover:text-gray-700" }, "\u2715")),
            React.createElement("div", { style: {
                    border: "1px solid #e5e7eb",
                    borderRadius: "0.25rem",
                    padding: "0.5rem 0.75rem",
                    backgroundColor: "white",
                    textAlign: "left",
                    color: "#222",
                    width: "360px",
                    maxHeight: "90vh",
                    overflow: "scroll",
                } },
                React.createElement("div", null,
                    React.createElement("span", { style: {
                            lineHeight: "1.25rem",
                        } }, "Topic")),
                React.createElement("div", { style: { marginTop: "0.25rem" } },
                    React.createElement("code", { style: {
                            lineHeight: "1.25rem",
                        } }, app.topic)),
                React.createElement("hr", { style: { margin: "0.5rem 0" } }),
                React.createElement("div", null,
                    React.createElement("span", { style: {
                            lineHeight: "1.25rem",
                        } }, "Connection")),
                React.createElement("div", { style: { marginTop: "0.25rem" } },
                    React.createElement("code", { style: {
                            lineHeight: "1.25rem",
                        } }, ws ? ws.sourceURL : "n/a"),
                    React.createElement("span", { style: { marginLeft: "0.5rem", color: "#6b7280" } },
                        "(",
                        !ws ? "No remote provided" : ws.isConnected() ? "Connected" : "Disconnected",
                        ")")),
                React.createElement("hr", { style: { margin: "0.5rem 0" } }),
                React.createElement("div", { style: {
                        lineHeight: "1.25rem",
                    } }, "Local merkle root"),
                React.createElement("div", { style: { marginTop: "0.25rem" } },
                    React.createElement("ul", { style: { listStyleType: "disc", paddingLeft: "1rem" } },
                        React.createElement("li", null, root !== null ? (React.createElement("code", { style: {
                                lineHeight: "1.25rem",
                            } }, root)) : (React.createElement("span", { style: { fontStyle: "italic" } }, "none"))))),
                React.createElement("hr", { style: { margin: "0.5rem 0" } }),
                React.createElement("div", { style: {
                        lineHeight: "1.25rem",
                    } }, "Local message heads"),
                React.createElement("div", { style: { marginTop: "0.25rem" } }, heads !== null ? (React.createElement("ul", { style: { listStyleType: "disc", paddingLeft: "1rem" } },
                    heads.map((head) => (React.createElement("li", { key: head },
                        React.createElement("code", { style: {
                                lineHeight: "1.25rem",
                            } }, head),
                        React.createElement("br", null),
                        React.createElement("span", { style: { color: "#6b7280" } },
                            "(clock: ",
                            MessageId.encode(head).clock,
                            ")")))),
                    heads.length === 0 && (React.createElement("li", null,
                        React.createElement("code", { style: {
                                lineHeight: "1.25rem",
                            } }, "None"),
                        React.createElement("br", null),
                        React.createElement("span", { style: { color: "#6b7280" } }, "(clock: 0)"))))) : (React.createElement("span", { style: { fontStyle: "italic" } }, "none"))),
                remoteInfo && (React.createElement("div", null,
                    React.createElement("hr", { style: { margin: "0.5rem 0" } }),
                    React.createElement("div", { style: {
                            lineHeight: "1.25rem",
                        } }, "Remote merkle root"),
                    React.createElement("div", { style: { marginTop: "0.25rem" } },
                        React.createElement("ul", { style: { listStyleType: "disc", paddingLeft: "1rem" } },
                            React.createElement("li", null,
                                React.createElement("code", null, remoteInfo.root))),
                        React.createElement("hr", { style: { margin: "0.5rem 0" } }),
                        React.createElement("div", null, "Remote message heads"),
                        React.createElement("ul", { style: { listStyleType: "disc", paddingLeft: "1rem" } },
                            remoteInfo.heads.map((head) => {
                                return (React.createElement("li", { key: head },
                                    React.createElement("code", { style: {
                                            lineHeight: "1.25rem",
                                        } }, head),
                                    React.createElement("br", null),
                                    React.createElement("span", { style: { color: "#6b7280" } },
                                        "(clock: ",
                                        MessageId.encode(head).clock,
                                        ")")));
                            }),
                            remoteInfo.heads.length === 0 && (React.createElement("li", null,
                                React.createElement("code", { style: {
                                        lineHeight: "1.25rem",
                                    } }, "None"),
                                React.createElement("br", null),
                                React.createElement("span", { style: { color: "#6b7280" } }, "(clock: 0)"))))),
                    React.createElement("hr", { style: { margin: "0.5rem 0" } }),
                    React.createElement("div", { style: {
                            lineHeight: "1.25rem",
                        } }, "Remote storage"),
                    React.createElement("div", { style: { marginTop: "0.25rem" } },
                        React.createElement("code", null, remoteInfo.database)))),
                ws && (React.createElement("div", null,
                    React.createElement("hr", { style: { margin: "0.5rem 0" } }),
                    React.createElement("button", { style: {
                            display: "block",
                            marginTop: "0.25rem",
                            color: "#3b82f6",
                            cursor: "pointer",
                        }, onClick: async () => {
                            if (ws === null || ws === undefined)
                                return;
                            setIsLoadingInfo(true);
                            try {
                                const response = await fetch(ws.sourceURL.replace(/^ws(s)?:\/\//, "http$1://") + "/api");
                                const data = await response.json();
                                setRemoteInfo(data);
                            }
                            catch (error) {
                                console.error("Failed to fetch remote info:", error);
                                setRemoteInfo({ error: "Failed to fetch remote info" });
                            }
                            setIsLoadingInfo(false);
                        } }, isLoadingInfo ? "Loading..." : "Refresh"),
                    React.createElement("a", { href: ws?.sourceURL.replace(/^ws(s)?:\/\//, "http$1://") + "/explorer", target: "_blank", rel: "noopener noreferrer", style: {
                            display: "block",
                            marginTop: "0.25rem",
                            color: "#3b82f6",
                            cursor: "pointer",
                        } }, "Manage remote \u2192"))))))));
};
