import React, { useCallback, useContext, useEffect, useRef, useState } from "react";
import { BrowserProvider } from "ethers";
import { SIWESigner } from "@canvas-js/signer-ethereum";
import { AuthContext } from "../AuthContext.js";
export const ConnectSIWE = ({ app }) => {
    const { sessionSigner, setSessionSigner, address, setAddress } = useContext(AuthContext);
    const [provider, setProvider] = useState(null);
    const [error, setError] = useState(null);
    const connect = useCallback(async (provider, isBrowserInit) => {
        if (app === null) {
            setError(new Error("app not initialized"));
            return;
        }
        if (provider === null) {
            setError(new Error("window.ethereum not found"));
            return;
        }
        setProvider(provider);
        const network = await provider.getNetwork();
        const signer = await provider
            .getSigner()
            .then((signer) => new SIWESigner({ signer, chainId: Number(network.chainId) }));
        if (!(await signer.hasSession(app.topic))) {
            if (isBrowserInit) {
                return;
            }
            else {
                const session = await signer.newSession(app.topic);
                await app.messageLog.append(session.payload, { signer: session.signer });
            }
        }
        const address = await signer.getDid();
        setAddress(address);
        const otherSigners = app.signers.getAll().filter((s) => !(s instanceof SIWESigner));
        otherSigners.forEach((s) => s.clearSession(app.topic));
        app.updateSigners([signer, ...otherSigners]);
        setSessionSigner(signer);
    }, [app, provider]);
    const initialRef = useRef(false);
    useEffect(() => {
        if (initialRef.current) {
            return;
        }
        initialRef.current = true;
        if (window.ethereum === undefined || window.ethereum === null) {
            setError(new Error("window.ethereum not found"));
            return;
        }
        // TODO: handle these more gracefully
        window.ethereum.on("chainChanged", (chainId) => window.location.reload());
        window.ethereum.on("accountsChanged", (accounts) => window.location.reload());
        const provider = new BrowserProvider(window.ethereum);
        setProvider(provider);
        // automatically log back in
        connect(provider, true);
    }, []);
    const disconnect = useCallback(async () => {
        if (app?.topic)
            sessionSigner?.clearSession(app?.topic);
        setAddress(null);
        setSessionSigner(null);
        const otherSigners = app.signers.getAll().filter((s) => !(s instanceof SIWESigner));
        app?.updateSigners([...otherSigners, new SIWESigner()]);
    }, [app, sessionSigner]);
    if (error !== null) {
        return (React.createElement("div", { className: "p-2 border rounded bg-red-100 text-sm" },
            React.createElement("code", null, error.message)));
    }
    else if (provider === null) {
        return (React.createElement("div", { className: "p-2 border rounded bg-gray-200" },
            React.createElement("button", { disabled: true }, "Loading...")));
    }
    else if (address !== null && sessionSigner instanceof SIWESigner) {
        return (React.createElement("button", { onClick: () => disconnect(), className: "p-2 border rounded hover:cursor-pointer hover:bg-gray-100 active:bg-gray-200" }, "Disconnect ETH wallet"));
    }
    else {
        return (React.createElement("button", { onClick: () => {
                connect(new BrowserProvider(window.ethereum));
            }, className: "p-2 border rounded hover:cursor-pointer hover:bg-gray-100 active:bg-gray-200" }, "Connect ETH wallet"));
    }
};
